import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.awt.event.KeyEvent; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class Visualisation extends PApplet {

/** @file Visualisation.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief main methods called on events, helper functions
 */

/**
 * method for initializing data loading
 * NOTE: order of addition specifies order of years in app
 */
public void prepareData() {
  years = new DataLoader();
  years.addAvailableYear("2009","../Data/ProcessedDataYear2009.csv");
  years.addAvailableYear("2010","../Data/ProcessedDataYear2010.csv");
  years.addAvailableYear("2011","../Data/ProcessedDataYear2011.csv");
  years.addAvailableYear("2012","../Data/ProcessedDataYear2012.csv");
  years.addAvailableYear("2013","../Data/ProcessedDataYear2013.csv");
  years.addAvailableYear("2014","../Data/ProcessedDataYear2014.csv");
  years.addAvailableYear("2015","../Data/ProcessedDataYear2015.csv");
  
  selectedYear = years.getLastYear();
}

// Main program
public void setup() {
  prepareData();
  controller = new Controller();
}

/** main draw method
 * increases time point, if animating
 * calls data redrawing, if needed
 */ 
public void draw() {
  if (helpDisplayed) {
    help.draw();
    return;
  }
  // if animating, increase counter (change based on speed)
  if (animate) {
    animateCounter += globalAnimationSpeed*100;
    if (animateCounter >= 50) {
      if((currentTimePoint+animateCounter/50) == 301 && stopAtAnimationEnd) {
        animate = false;
        currentTimePoint = 301;
      } else {
        currentTimePoint = (currentTimePoint+animateCounter/50) % 301;
      }
      animateCounter = 0;
      controller.drawControlPanel();
      redrawData = true;
    }
  }
  
  // redraw data, if needed
  if (redrawData) {
    years.get(selectedYear).drawDataBackground();
    years.get(selectedYear).drawData();
    years.get(selectedYear).drawDataAxes();
    years.get(selectedYear).drawTeamInfo();
    redrawData = false;
  }
}

/** main mouse click processing method
 * passes click processing to controller or year, depending on location
 */
public void mousePressed() {
  if (mouseY > screenHeight-controlPanelHeight) {
    controller.changeSettingsViaClick(mouseX, mouseY);
  } else {
    years.get(selectedYear).processClick(mouseX, mouseY);
  }
}

/** main keyboard processing method
 * passes key processing to controller
 */ 
public void keyPressed() {
  controller.changeSettingsViaKey();
}

/** main mouse drag processing method
 * if draggable item is selected, pass processing as click to controller
 * adjust click position first
 */
public void mouseDragged() {
  if (dragging) {
    controller.changeSettingsViaClick(mouseX-dragXOffset, mouseY);
  }
}

/** main mouse reseasing method
 * deactivate selection of draggable items
 */
public void mouseReleased() {
  dragging = false;
}
/** @file Controller.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief controller object, adjusts settings and manages control panel drawing
 */
 


class Controller {
  
  // slider change if RIGHT/LEFT arrows are presed
  float keySliderShift = 0.1f;
  // minutes passed if UP/DOWN arrows are pressed 
  int keyTimeShift = 1;
  
  // height of the slider bar
  int sliderHeight = 20;
  // part of the bar occupied by actual slider (its relative width)
  float sliderMoverRatio = 0.1f;
  // pixel space between slider and bar border
  int sliderOffset = 4;
  
  // tick-box size constant
  int boxSide = 16;
  // y position of top bounding box for button choosers
  int topYbuttons = screenHeight-controlPanelHeight+70-12;
  // margin applied for time and velocity slider
  int sliderMargin = 30;
  // x position of center of the first play/pause button
  int buttonsXbegin = 350+130+boxSide/2;
  // y position of center of the top of speed slider
  int speedSliderY = screenHeight-controlPanelHeight+sliderHeight+80;
  // length between play/pause button centers
  int buttonSpacing = (int)(1.5f*boxSide);
  
  // button constants
  static final int BUTTON_PLAY = 0;
  static final int BUTTON_PAUSE = 1;
  static final int BUTTON_HOME = 2;
  static final int BUTTON_END = 3;
  static final int BUTTON_NEXT = 4;
  static final int BUTTON_PREV = 5;

  /** Controller constructor
   * adjusts screen settings
   * loads images and fonts to global variables
   * displays control panel
   */
  Controller() {
    size(screenWidth, screenHeight);
    frameRate(25);
    smooth();
    noStroke();
    // load background
    bgImage = loadImage(bgImagePath);
    // load fonts
    fonts = new PFont[4];
    for (int i = 0; i < 4; i++) {
      fonts[i] = loadFont(fontPaths[i]);
    }
    // load logos
    logoImages = new PImage[3];
    for (int i = 0; i < 3; i++) {
      logoImages[i] = loadImage(logoImagePaths[i]);
    }
    // draw control panel and data
    drawControlPanel();
    redrawData = true;
  }
  
  /** method for switching years */
  public void handleYearChange(boolean toNextYear) {
    currentTeamLeft = -1;
    currentTeamRight = -1;
    selectedYear = (toNextYear) ? years.getNext(selectedYear) : years.getPrev(selectedYear);
  }

  /** general method for mouse clicking the control panel
   * calls appropriate subrutine
   * changing settings redraws the control panel and data
   * assumtion: click was placed somewhere in the control area
   */
  public void changeSettingsViaClick(int x, int y) {
    boolean clickMeaningful = false;
    // help button click
    if (isIn(x, screenWidth-210, screenWidth-210+180) &&
        isIn(y, screenHeight-60, screenHeight-60+30)) {
          clickMeaningful = true;
      helpDisplayed = !helpDisplayed;
    }
    if (helpDisplayed) {
      return;
    }
    // data slider click
    if (y < screenHeight-controlPanelHeight+sliderHeight) {
      clickMeaningful = true;
      dataSliderPosition = changeSliderPosition(x, screenWidth, dataSliderPosition);
    }
    // year buttons click
    int yearTopBox = (int) (screenHeight-controlPanelHeight+70);
    if ((isIn(x, 20, 40) && isIn(y, topYbuttons, topYbuttons+16)) ||
        (isIn(x, 130, 150) && isIn(y, topYbuttons, topYbuttons+16))) {
       clickMeaningful = true;
       handleYearChange((isIn(x, 130, 150) && isIn(y, topYbuttons, topYbuttons+16)));
     }
    // category selector click
    if (isIn(x, 180, 180+140) && 
        isIn(y, topYbuttons, topYbuttons+24.9f*3)) {
      clickMeaningful = true;
      selectedCategories[(y-topYbuttons)/25] = !selectedCategories[(y-topYbuttons)/25];
    }
    // time slider click
    int sliderTopPos = screenHeight-controlPanelHeight+sliderHeight+sliderMargin/2;
    if (isIn(x, 350+sliderMargin, screenWidth-sliderMargin) && 
        isIn(y, sliderTopPos, sliderTopPos+sliderHeight)) {
      clickMeaningful = true;
      int relativeX = x-(350+sliderMargin);
      int barWidth = screenWidth-350-2*sliderMargin;
      currentTimePoint = (int)(301*changeSliderPosition(relativeX, barWidth, (float)currentTimePoint/301));
    }
    // speed slider click
    if (isIn(x, screenWidth-300+sliderMargin, screenWidth-sliderMargin) && 
        isIn(y, speedSliderY, speedSliderY+sliderHeight)) {
      clickMeaningful = true;
      int relativeX = x-(screenWidth-300+sliderMargin);
      globalAnimationSpeed = changeSliderPosition(relativeX, 300-2*sliderMargin, globalAnimationSpeed);
    }
    // animation start/stop/home/end click
    if (isIn(x, buttonsXbegin-buttonSpacing/2, buttonsXbegin+5*buttonSpacing/2) &&
        isIn(y, speedSliderY, speedSliderY+buttonSpacing) ) {
      clickMeaningful = true;
      switch ((int)((x-buttonsXbegin+buttonSpacing/2)/buttonSpacing)) {
        case 0: currentTimePoint = 0; break;
        case 1: animate = !animate; animateCounter = 0; break;
        case 2:
        case 3: currentTimePoint = 301; break;
        default: break;
      }
    }
    // stop at animation end click
    if (isIn(x, buttonsXbegin + 3.5f*buttonSpacing, buttonsXbegin+7*buttonSpacing) &&
        isIn(y, speedSliderY, speedSliderY+buttonSpacing) ) {
      clickMeaningful = true;
      stopAtAnimationEnd = !stopAtAnimationEnd;
    }
    // if we clicked something meaningfull, redraw everything
    if (clickMeaningful) {
      drawControlPanel();
      redrawData = true;
    }
  }

  /** general method for keys pressed
   */
  public void changeSettingsViaKey() {
    if (helpDisplayed) {
      if (keyCode == KeyEvent.VK_H) {
        helpDisplayed = false;
        // redraw everything
        drawControlPanel();
        redrawData = true;
      }
      return;
    }
    switch (keyCode) {
      case KeyEvent.VK_PAGE_DOWN:
      dataSliderPosition = min(dataSliderPosition+keySliderShift,1); break;
      case KeyEvent.VK_PAGE_UP:
      dataSliderPosition = max(dataSliderPosition-keySliderShift,0); break;
      case KeyEvent.VK_HOME:
      dataSliderPosition = 0; break;
      case KeyEvent.VK_END:
      dataSliderPosition = 1; break;
      case UP:
      currentTimePoint = min(currentTimePoint+keyTimeShift, 301); break;
      case DOWN:
      currentTimePoint = max(currentTimePoint-keyTimeShift, 0); break;
      case ENTER:
      case KeyEvent.VK_SPACE:
      animate = !animate; animateCounter = 0; break;
      case LEFT:
      currentTimePoint = 0; break;
      case RIGHT:
      currentTimePoint = 301; break;
      case KeyEvent.VK_N:
      handleYearChange(true);
      break;
      case KeyEvent.VK_P:
      handleYearChange(false);
      break;
      case KeyEvent.VK_S:
      globalAnimationSpeed = max(0, globalAnimationSpeed - 0.02f);
      break;
      case KeyEvent.VK_F:
      globalAnimationSpeed = min(1, globalAnimationSpeed + 0.02f);
      break;
      case KeyEvent.VK_E:
      stopAtAnimationEnd = !stopAtAnimationEnd;
      break;
      case KeyEvent.VK_H:
      helpDisplayed = !helpDisplayed;
      break;
      case KeyEvent.VK_1:
      selectedCategories[0] = !selectedCategories[0]; 
      break;
      case KeyEvent.VK_2:
      selectedCategories[1] = !selectedCategories[1]; 
      break;
      case KeyEvent.VK_3:
      selectedCategories[2] = !selectedCategories[2]; 
      break;
      default: return; // not meaningful, do not redraw
    }
    // redraw everything
    drawControlPanel();
    redrawData = true;
  }

  /** draw the control panel including data slider
   */
  public void drawControlPanel() {
    // draw background
    int numImagesWidth = ceil((float)screenWidth/bgImage.width);
    for (int i = 0; i < numImagesWidth; i++) {
      image(bgImage, i*bgImage.width, screenHeight-controlPanelHeight);
    }

    // draw data slider
    drawSlider(0, screenHeight-controlPanelHeight, screenWidth, dataSliderPosition);
    
    // draw year chooser (0-150 pixels from left)
    fill(brownDark);
    textFont(fonts[1]);
    String text = "ro\u010dn\u00edk";
    text(text, 85-textWidth(text)/2, screenHeight-controlPanelHeight+sliderHeight+20+textAscent()/2);
    textFont(fonts[0]);
    text = years.get(selectedYear).name;
    int yearTopBox = (int) (screenHeight-controlPanelHeight+70);
    text(text, 65, yearTopBox);
    drawButton(30, yearTopBox - 5, BUTTON_PREV);
    drawButton(140, yearTopBox - 5, BUTTON_NEXT);
    
    // draw categories chooser (150-350 pixels from left)
    fill(brownDark);
    textFont(fonts[1]);
    text = "kategorie";
    text(text, 250-textWidth(text)/2, screenHeight-controlPanelHeight+sliderHeight+20+textAscent()/2);
    textFont(fonts[0]);
    for (int i = 0; i < 3; i++) {
      drawBox(195, screenHeight-controlPanelHeight+70+i*25, selectedCategories[i]);
      switch (i) {
        case 0: text = "st\u0159edo\u0161kol\u00e1ci"; break;
        case 1: text = "vysoko\u0161kol\u00e1ci"; break;
        case 2: text = "ostatn\u00ed"; break;
      }
      text(text, 215, screenHeight-controlPanelHeight+70+i*25+textAscent()/2);
    }
    
    // draw time slider (350-screenWidth pixels from left)
    drawSlider(350+sliderMargin, screenHeight-controlPanelHeight+sliderHeight+sliderMargin/2, 
               screenWidth-350-2*sliderMargin, (float)currentTimePoint/301);
    fill(brownDark);
    textFont(fonts[0]);
    for(int hour = 0; hour <= 5; hour++) {
      text = "" + (hour+15) + ":00";
      int position = (int)map(hour, 0, 5, 350+sliderMargin, screenWidth-sliderMargin-textWidth(text));
      text(text, position, screenHeight-controlPanelHeight+2*sliderHeight+3*sliderMargin/4+textAscent());
    } 
    
    // draw speed slider ((screenWidth-300)-screenWidth pixels from left)
    fill(brownDark);
    textFont(fonts[1]);
    text = "rychlost";
    text(text, screenWidth-300+sliderMargin/2-textWidth(text), speedSliderY+textAscent());
    drawSlider(screenWidth-300+sliderMargin, speedSliderY, 300-2*sliderMargin, globalAnimationSpeed);
    
    // draw animation buttons
    fill(brownDark);
    textFont(fonts[1]);
    text = "\u010dasov\u00e1 osa";
    text(text, 350+sliderMargin, speedSliderY+textAscent());
    drawButton(buttonsXbegin, speedSliderY+boxSide/2, BUTTON_HOME);
    drawButton(buttonsXbegin + 2*buttonSpacing, speedSliderY+boxSide/2, BUTTON_END);
    if (animate) {
      drawButton(buttonsXbegin+buttonSpacing, speedSliderY+boxSide/2, BUTTON_PAUSE);
    } else {
      drawButton(buttonsXbegin+buttonSpacing, speedSliderY+boxSide/2, BUTTON_PLAY);
    }
    
    drawBox(buttonsXbegin + 4*buttonSpacing, speedSliderY+boxSide/2, stopAtAnimationEnd);
    String stopAtAnimationEnd = "skon\u010dit";
    text(stopAtAnimationEnd, buttonsXbegin + 5*buttonSpacing, speedSliderY+textAscent());
    
    stroke(brownMedium);
    rect(screenWidth-210, screenHeight-60, 180, 30);
    text("n\u00e1pov\u011bda & z\u00e1sluhy", screenWidth-195, screenHeight-52+textAscent());
  }
  
  /** general method for slider drawing
   * @param x              x of the upper left corner of the entire bar
   * @param y              y of the upper left corner of the entire bar
   * @param barWidth       width of the entire bar
   * @param position       slider position given in [0,1] including
   */
  public void drawSlider(int x, int y, int barWidth, float position) {
    // draw bar
    noStroke();
    fill(red(brownLight), green(brownLight), blue(brownLight), 64);
    rect(x, y, barWidth, sliderHeight);
    // compute relative x slider position
    fill(red(brownLight), green(brownLight), blue(brownLight), 128);
    int sliderWidth = (int)(barWidth*sliderMoverRatio);
    float sliderCenter = map(position, 0, 1, sliderWidth/2, barWidth-sliderWidth/2);
    rect(x+sliderCenter-sliderWidth/2+sliderOffset, y+sliderOffset, 
         sliderWidth-2*sliderOffset, sliderHeight-2*sliderOffset);
  }
  
  /** general method for slider position changing
   * enables dragging if needed
   * @param relativeX      position of the click from the left side of slider bar
   * @param barWidth       width of the entire bar
   * @param position       original slider position given in [0,1] including
   * @return               new slider position in [0,1] including
   */
  public float changeSliderPosition(int relativeX, int barWidth, float position) {
    int sliderWidth = (int)(barWidth*sliderMoverRatio);
    int currentSliderPosition = (int)map(position, 0, 1, sliderWidth/2, barWidth-sliderWidth/2);
    // enable dragging if necessary
    if (!dragging 
        && relativeX > currentSliderPosition-sliderWidth/2 
        && relativeX < currentSliderPosition+sliderWidth/2 ) {
      dragging = true;
      dragXOffset = relativeX - currentSliderPosition;
      return position;
    }
    // if clicked outside if the slider, reposition
    relativeX = constrain(relativeX, sliderWidth/2, barWidth-sliderWidth/2);
    return map(relativeX, sliderWidth/2, barWidth-sliderWidth/2, 0, 1);
  }
  
  /** general method for tick-box drawing
   * @param x         x coordinate of box center
   * @param y         y coordinate of box center
   * @param ticked    should tick be placed inside?
   */
  public void drawBox(int x, int y, boolean ticked) {
    strokeWeight(2);
    stroke(brownDark);
    noFill();
    rect(x-boxSide/2, y-boxSide/2, boxSide, boxSide);
    strokeWeight(5);
    if (ticked) {
      line(x-boxSide/2, y, x-boxSide/8, y+boxSide/3);
      line(x+boxSide/2, y-boxSide/2, x-boxSide/8, y+boxSide/3);
    }
    noStroke();
  }
  
  /** general method for function circle drawing
   * @param x      x coordinate of top left corner  
   * @param y      y coordinate of top left corner
   * @param type   should the circle should be "ticked"
   */
  public void drawCircle(int x, int y, boolean selected) {
    int strokeWeight = 2;
    strokeWeight(strokeWeight);
    stroke(brownDark);
    noFill();
    float radius = sqrt(pow(boxSide,2)+pow(boxSide,2))/2;
    ellipse(x-boxSide/2+radius-strokeWeight*3/2, y-boxSide/2+radius-strokeWeight*3/2, boxSide, boxSide);
    if (selected) {
      strokeWeight(3);
      fill(brownDark);
      ellipse(x-boxSide/2+radius-strokeWeight*3/2, y-boxSide/2+radius-strokeWeight*3/2, boxSide*2/5, boxSide*2/5);
    }
    noStroke();
  }
  
  /** general method for function button drawing
   * @param x      x coordinate of button center
   * @param y      y coordinate of button center
   * @param type   button type (see constants at top)
   */
  public void drawButton(int x, int y, int type) {
    strokeWeight(2);
    stroke(brownDark);
    noFill();
    rect(x-boxSide/2, y-boxSide/2, boxSide, boxSide);
    fill(brownDark);
    noStroke();
    switch (type) {
      case BUTTON_PLAY: 
      triangle(x-boxSide/4,y-2*boxSide/6, x-boxSide/4,y+2*boxSide/6, x+2*boxSide/6,y); break;
      case BUTTON_PAUSE:
      rect(x-boxSide/4, y-2*boxSide/6, boxSide/4, 2*boxSide/3);
      rect(x+boxSide/8, y-2*boxSide/6, boxSide/4, 2*boxSide/3);
      break;
      case BUTTON_HOME:
      triangle(x+2*boxSide/6,y-2*boxSide/6, x+2*boxSide/6,y+2*boxSide/6, x-boxSide/6,y);
      rect(x-boxSide/6-boxSide/6, y-2*boxSide/6, boxSide/6, 2*boxSide/3);
      break;
      case BUTTON_END:
      triangle(x-2*boxSide/6,y-2*boxSide/6, x-2*boxSide/6,y+2*boxSide/6, x+boxSide/6,y);
      rect(x+boxSide/6, y-2*boxSide/6, boxSide/6, 2*boxSide/3);
      break;
      case BUTTON_NEXT:
      triangle(x-2*boxSide/6,y-2*boxSide/6, x-2*boxSide/6,y+2*boxSide/6, x+2*boxSide/6,y);
      break;
      case BUTTON_PREV:
      triangle(x+2*boxSide/6,y-2*boxSide/6, x+2*boxSide/6,y+2*boxSide/6, x-2*boxSide/6,y);
      break;
      default: break;
    }
    noFill();
  }
}

/** @file DataLoader.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief load and parse data from csv file which is preprocessed by script, see DataPreprocessing.py
 */
class DataLoader {  
  private HashMap<String, String> availableYears = new HashMap<String, String>();
  private HashMap<String, Year> years = new HashMap<String,Year>();
  private ArrayList<String> yearsOrder = new ArrayList<String>();
  
  public void addAvailableYear(String year, String filePath) {
    availableYears.put(year, filePath);
    yearsOrder.add(year);
  }
  
  public Year get(String year) {
   if (!availableYears.containsKey(year)) {
      return null; 
    }
    if (!years.containsKey(year)) {
      load(year, availableYears.get(year));
    }
    return years.get(year);
  }
  
  public String getLastYear() {
    if (yearsOrder.isEmpty()) {
      return null;
    }
    return yearsOrder.get(yearsOrder.size()-1);
  }
  
  public String getNext(String current) {
    for (int i = 0; i < yearsOrder.size(); i++) {
      if (yearsOrder.get(i).equals(current)) {
        return yearsOrder.get((i + 1) % yearsOrder.size());
      }
    }
    return current;
  }
  
  public String getPrev(String current) {
    for (int i = 0; i < yearsOrder.size(); i++) {
      if (yearsOrder.get(i).equals(current)) {
        return yearsOrder.get(((i + yearsOrder.size()) - 1) % yearsOrder.size());
      }
    }
    return current;
  }
  
  private void load(String year, String filePath) {
    Year tempYear = new Year();
    tempYear.name = year;
    //BufferedReader reader = createReader(filePath);
    String lines[] = loadStrings(filePath);
    String line;
    String[] temp;
    Team tempTeam;
    for (int j = 0 ; j < lines.length; j++) {
      line = lines[j];
      temp = split(line,"$");
      tempTeam = new Team();
      tempTeam.id = Integer.parseInt(temp[0]);
      tempTeam.name = temp[1];
      tempTeam.parseAndSetCategory(temp[2]);
      int OFFSET = 3;
      State tempState;
      // Load all states into array
      for(int i = 0; i < 302; i++) {
        tempState = new State();
        tempState.numberOfLogical = Integer.parseInt(temp[OFFSET + 1 + i*9]);
        tempState.numberOfProgramming = Integer.parseInt(temp[OFFSET + 2 + i*9]);
        tempState.numberOfIdea = Integer.parseInt(temp[OFFSET + 3 + i*9]);

        tempState.scoreLogical = Integer.parseInt(temp[OFFSET + 4 + i*9]);
        tempState.scoreProgramming = Integer.parseInt(temp[OFFSET + 5 + i*9]);
        tempState.scoreIdea = Integer.parseInt(temp[OFFSET + 6 + i*9]);

        tempState.bonus = Integer.parseInt(temp[OFFSET + 7 + i*9]);

        tempState.penalisation = Integer.parseInt(temp[OFFSET + 8 + i*9]);
        
        tempTeam.states[i] = tempState;
      }
      
      tempYear.teams.add(tempTeam);
    }
    years.put(year, tempYear);
  }
}
/** @file Globals.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief global settings, constants and pointers
 */

// SCREEN CONSTANTS
// Screen resolution, change at will (don't make it smaller than 700x1024)
int screenHeight = 700;
int screenWidth = 1024;
// Height of the panel, this should not be changed 
int controlPanelHeight = 200;

// CURRENT SETTINGS
// selected year as a string (see DataLoader and prepareData())
String selectedYear = null;
// displayed categories [stredoskolaci, vysokoskolaci, ostatni]
boolean[] selectedCategories = {true, true, true};
// time point in [0,301] including (used as index to states array in team)
int currentTimePoint = 301;
// is the animation going?
boolean animate = false;
boolean stopAtAnimationEnd = true;
// should be help displayed?
boolean helpDisplayed = false;
// animation speed in [0,1] including
float globalAnimationSpeed = 0.35f;
// currently selected team
int currentTeamLeft = -1;
int currentTeamRight = -1;

// GLOBAL OBJECTS
// storage for all available and already loaded years
DataLoader years;
// global singleton controller object 
Controller controller;
// global object with help
Help help = new Help();

// GLOBAL VARIABLES FOR DRAWING
// position of data slider in [0,1] including
float dataSliderPosition = 0;
// counter for animation speed, new minute as advanced for every 100
int animateCounter = 0;
// should data area be redrawn?
boolean redrawData = true;
// is draggable item selected?
boolean dragging = false;
// dragging offset (how far from center are we holding the slider?)
int dragXOffset = 0;

// GLOBAL DISPLAY CONSTANTS
// width of team column in pixels
int teamColumnWidth = 30;
// margin between 2 team columns in pixels
int teamColumnMargin = 20;
// margin between team detail info box
int teamDetailMargin = 10;

// DATA AREA CONSTANTS
  int dataBottomMargin = 90;
  int dataTopMargin = 120;
  int dataLeftMargin = 100;
  int legendRightMargin = 500;
  int legendTopMargin = 20;
  // width of horisontal axes
  int axesStroke = 4;
  // number of point between 2 horisontal axes
  int axesPointsInterval = 5000;

// COLORS AND DESIGN
int brownDark = color(45,22,15);
int brownMedium = color(226, 201, 143);
int brownLight = color(216,181,94);
int green = color(26, 110, 0);
int red = color(186,13,13);
// background image
String bgImagePath = "images/background.png";
PImage bgImage;
// logos: InterLoS, FI, MU
String[] logoImagePaths = { "images/logo-los.png", "images/logo-fi.png", "images/logo-mu.png" };
PImage[] logoImages;
// used fonts: OpenSans-15, OpenSans-Bold-15, OpenSans-Bold-35, , OpenSans-13
String[] fontPaths = {"fonts/OpenSans-15.vlw", "fonts/OpenSans-Bold-15.vlw", "fonts/OpenSans-Bold-35.vlw", "fonts/OpenSans-13.vlw"};
PFont[] fonts;
/** @file Help.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief helper object which contains all text about help and credits
 */
 
class Help {
  
  public void draw() {
    background(brownMedium);
    textFont(fonts[2]);
    fill(brownDark);
    text("N\u00e1pov\u011bda", 80, 80);
    text("Z\u00e1sluhy", 600, 80);
    
    textFont(fonts[0]);
    String text = "";
    text +="Ro\u010dn\u00edky\n";
    text += "    p\n";
    text += "    n\n";
    text += "Kategorie\n";
    text += "    1\n";
    text += "    2\n";
    text += "    3\n";
    text += "T\u00fdmy\n";
    text += "     home\n";
    text += "     end\n";
    text += "     page up\n";
    text += "     page down\n";
    text += "     lev\u00e9/prav\u00e9 tla\u010d\u00edtko\n";
    text += "Animace\n";
    text += "    mezern\u00edk/enter\n";
    text += "    \u0161ipka vlevo/vpravo\n";
    text += "    \u0161ipka dol\u016f/nahoru\n";
    text += "    s\n";
    text += "    f\n";
    text += "    e\n";
    text += "N\u00e1pov\u011bda\n";
    text += "    h\n";
    text(text, 80, 150);
    text = "";
    text +="\n";
    text += "p\u0159edchoz\u00ed ro\u010dn\u00edk\n";
    text += "n\u00e1sleduj\u00edc\u00ed ro\u010dn\u00edk\n";
    text += "\n";
    text += "zahrnout/schovat vysoko\u0161kol\u00e1ky\n";
    text += "zahrnout/schovat st\u0159edo\u0161kol\u00e1ky\n";
    text += "zahrnout/shocvat ostatn\u00ed\n";
    text += "T\u00fdmy\n";
    text += "posun na za\u010d\u00e1tek\n";
    text += "posun na konec\n";
    text += "posun na p\u0159edchoz\u00ed str\u00e1nku\n";
    text += "posun na dal\u0161\u00ed str\u00e1nku\n";
    text += "zobraz\u00ed detail t\u00fdmu\n";
    text += "\n";
    text += "p\u0159ehr\u00e1n\u00ed/pauza\n";
    text += "skok na za\u010d\u00e1tek/konec\n";
    text += "skok na p\u0159edchoz\u00ed/n\u00e1sleduj\u00edc\u00ed minutu\n";
    text += "sn\u00ed\u017eit rychlost\n";
    text += "zv\u00fd\u0161it rychlost\n";
    text += "skon\u010dit na konci\n";
    text += "\n";
    text += "otev\u0159\u00edt/zav\u0159\u00edt\n";
    text(text, 270, 150);
    
    text = "";
    text += "Auto\u0159i\n";
    text += "   - Martin Ukrop\n";
    text += "   - Jan Dr\u00e1bek\n\n";
    text += "Webov\u00e1 str\u00e1nka sout\u011b\u017ee\n";
    text += "    interlos.fi.muni.cz\n\n";
    text += "Tento projekt by nevzniknul neb\u00fdt p\u0159edm\u011btu\n";
    text += "PA055 Vizualizace komplexn\u00edch dat vyu\u010dovan\u00e9ho\n";
    text += "na Masarykov\u011b univerzit\u011b. Vyu\u010duj\u00edc\u00edmu p\u0159edm\u011btu,\n";
    text += "Mateji Lexovi, t\u00edmto d\u011bkujeme za tuto p\u0159\u00edle\u017eitost.\n\n";
    text += "Takt\u00e9\u017e chceme pod\u011bkovat v\u0161em organiz\u00e1tor\u016fm sout\u011b\u017ee\n";
    text += "InterLoS i sout\u011b\u017e\u00edc\u00edm, bez kter\u00fdch nemohla b\u00fdt tato\n";
    text += "vizualizace takov\u00e1 jak\u00e1 je.";
    
    text(text, 600, 150);
    
    stroke(brownLight);
    fill(brownMedium);
    rect(screenWidth-210, screenHeight-60, 180, 30);
    fill(brownDark);
    textFont(fonts[1]);
    text("zav\u0159\u00edt", screenWidth-145, screenHeight-52+textAscent());
  }
}
/** @file Helpers.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief helper functions
 */

/** interval checking, helper method
 * @param num  value to check
 * @param min  lower bound of interval to check
 * @param max  upper bound of interval to check
 */
public boolean isIn(float num, float min, float max) {
  return (num >= min && num <= max);
}

/**
 * general method for shortening text with elipsis
 * @param input      string to be shortened
 * @param maxLength  maximal length of shortened text
 * @return           shortened text
 */
public String truncate(String input, int maxLength) {
  if(input.length() > maxLength) {
    return input.substring(0,maxLength-3) + "...";
  }
  return input;
}

/** general method for function cross drawing
 * @param x    x coordinate of top left corner
 * @param y    y coordinate of top left corner
 */
public void drawCross(int x, int y, int boxSide) {
  strokeWeight(2);
  stroke(brownDark);
  line(x, y, x+boxSide/2, y+boxSide/2);
  line(x+boxSide/2, y, x, y+boxSide/2);
  noStroke();
}
/** @file State.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief class for representing state of team in fixed minute from the beginning of game
 */

class State {
  int numberOfLogical;
  int numberOfProgramming;
  int numberOfIdea;
  
  int scoreLogical;
  int scoreProgramming;
  int scoreIdea;
  
  int bonus;
  
  int penalisation;
  
  public int getTotalScore() {
    return scoreLogical + scoreProgramming + scoreIdea + bonus - penalisation;
  }
}
/** @file Team.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief Class which holds informations about team and all states during the game.
 *        Manages data area drawing.
 */
 
class Team {
  String name;
  int id;
  int category;
  
  static final int COLLEGE = 1;
  static final int HIGH_SCHOOL = 0;
  static final int OTHER = 2;
  
  State[] states = new State[302];  // Key = minute from beginning of game
  
  /** returns level (y coordinate) of this team to render comparison line */
  public float getLevel(int y, int maxScore, int maxHeight) {
      State currentState = states[currentTimePoint];
      float ratio = getRatio(maxScore, maxHeight);
      return ceil(y-currentState.scoreLogical/ratio - currentState.scoreProgramming/ratio - currentState.scoreIdea/ratio - currentState.bonus/ratio);
  }
  
  /** returns ratio according to */
  private float getRatio(int maxScore, int maxHeight) {
    State currentState = states[currentTimePoint];
    int score = currentState.getTotalScore();
    int potentialScore = score + currentState.penalisation;
    float colHeight = map(score, 0, maxScore, 0, maxHeight);
    return score/colHeight;
  }
  
  /** draw team column
   * @param x          x of lower left corner of bar
   * @param y          y of lower left corner of bar
   * @param maxScore   score corresponding to maxHeight
   * @param maxHeight  maximum allowed bar height
   */
  public void draw(int x, int y, int maxScore, int maxHeight) {
    State currentState = states[currentTimePoint];
    //int score = currentState.getTotalScore();
    //int potentialScore = score + currentState.penalisation;
    //float colHeight = map(score, 0, maxScore, 0, maxHeight);
    float ratio = getRatio(maxScore, maxHeight);
    
    fill(red(brownLight), green(brownLight), blue(brownLight), 100);
    rect(x, ceil(y-currentState.scoreLogical/ratio), teamColumnWidth, ceil(currentState.scoreLogical/ratio)); // Overlaps are less problems than holes between -> ceil
 
    fill(red(brownLight), green(brownLight), blue(brownLight), 170);
    rect(x, ceil(y-currentState.scoreLogical/ratio - currentState.scoreProgramming/ratio), teamColumnWidth, ceil(currentState.scoreProgramming/ratio));
    
    fill(red(brownLight), green(brownLight), blue(brownLight), 255);
    rect(x, ceil(y-currentState.scoreLogical/ratio - currentState.scoreProgramming/ratio - currentState.scoreIdea/ratio), teamColumnWidth, ceil(currentState.scoreIdea/ratio));
    
    fill(red(green), green(green), blue(green), 150);
    rect(x, ceil(y-currentState.scoreLogical/ratio - currentState.scoreProgramming/ratio - currentState.scoreIdea/ratio - currentState.bonus/ratio), teamColumnWidth, ceil(currentState.bonus/ratio));

    fill(red(red), green(red), blue(red), 150);
    float yPos = ceil(y-currentState.scoreLogical/ratio - currentState.scoreProgramming/ratio - currentState.scoreIdea/ratio - currentState.bonus/ratio);
    rect(x, yPos, teamColumnWidth, currentState.penalisation/ratio);

    // Render notification about score changes above the team bar
    if(currentTimePoint == 301) {  // Ignore when on the end of animation
      return;
    }
    int[] changes = getChanges();
    int notificationRendered = 0;
    textFont(fonts[3]);
    textAlign(CENTER);
    for(int i = 0; i < changes.length; i++) {
      if(changes[i] != 0) {
        if(changes[i] < 0) {
          fill(red(red), green(red), blue(red), map(i, changes.length, 0, 40, 255));
        } else {
          fill(red(green), green(green), blue(green), map(i, changes.length, 0, 40, 255));
        }
        text(changes[i], x+teamColumnWidth/2, yPos-5-15*notificationRendered);
        notificationRendered++;
      }
    }
    textAlign(LEFT);
    textFont(fonts[0]);
  }
  
  /**
   * Resolves history length on basis of globalAnimationSpeed (faster means longer history).
   * @return  history length to be used in current settings
   */
  public int computeHistoryLength() {
    return floor(map(globalAnimationSpeed*10, 0, 10, 1, 25));
  }
  
  /**
   * Get changes between minutes.
   * @return  Array with changes between minutes going for computeHistoryLength() to the history
   */
  public int[] getChanges() {
    int historyLength = computeHistoryLength();
    int[] changes = new int[historyLength];
    int last = 0;
    for(int i = currentTimePoint; i > 0 && currentTimePoint - i < historyLength; i--) {
      changes[last++] = (getChangeBetweenStates(i, i-1));
    } 
    return changes;
  }
  
  /**
   * Retrieve score change between two given (and existing) states
   * @return  Score difference
   */
  public int getChangeBetweenStates(int currentID, int previousID) {
    State current = states[currentID];
    State previous = states[previousID];
    int diff = 0;
    diff += current.scoreProgramming - previous.scoreProgramming;
    diff += current.scoreLogical - previous.scoreLogical;
    diff += current.scoreIdea - previous.scoreIdea;
    diff += current.bonus - previous.bonus;
    diff -= current.penalisation - previous.penalisation;
    return diff;
  }
  
  /**
   * Draw box with team info in current timepoint.
   * @param XOffset  where to start box
   * @param x        X coordinate of space for detail team boxes info
   * @param y        Y coordinate of space for detail team boxes info
   */
  public void drawInfo(int XOffset, int x, int y) {
    int boxLeft = x + XOffset;
    int boxTop = y;
    fill(red(brownLight), green(brownLight), blue(brownLight), 64);
    rect(boxLeft, boxTop, 440, 70);
    if(XOffset == 0) {
      fill(green);
    } else {
      fill(red);
    }
    ellipse(x+XOffset+10, y+15, 10, 10);
    fill(brownDark);
    // Team name
    textFont(fonts[1]);
    String truncatedName = truncate(name,20);
    text(truncatedName, boxLeft+20, boxTop+20);
    // Team category
    float nameWidth = textWidth(truncatedName);
    textFont(fonts[0]);
    text("(" + getCategoryName() + ")", boxLeft+40+nameWidth, boxTop+20);
    // Tasks and penalisation
    text("Logick\u00e9:", boxLeft+20, boxTop+40);
    text("Programovac\u00ed:", boxLeft+20, boxTop+60);
    text("\u0160ifrovac\u00ed:", boxLeft+220, boxTop+40);
    text("Penalizace:", boxLeft+220, boxTop+60);

    State currentState = states[currentTimePoint];
    textAlign(RIGHT);
    text("(" + currentState.numberOfLogical + ")", boxLeft+210, boxTop+40);
    text("(" + currentState.numberOfProgramming + ")", boxLeft+210, boxTop+60);
    text("(" + currentState.numberOfIdea + ")", boxLeft+400, boxTop+40);
    text("(" + currentState.penalisation/30 + ")", boxLeft+400, boxTop+60);
    
    text(currentState.scoreLogical + " b", boxLeft+180, boxTop+40);
    text(currentState.scoreProgramming + " b", boxLeft+180, boxTop+60);
    text(currentState.scoreIdea + " b", boxLeft+360, boxTop+40);
    text(currentState.penalisation + " b", boxLeft+360, boxTop+60);
    
    textFont(fonts[1]);
    text(currentState.getTotalScore() + " b", boxLeft+360, boxTop+20);
    textFont(fonts[0]);
    textAlign(LEFT);
    
    // Closing cross
    drawCross(boxLeft+415, boxTop+8, 25);
  }
  
  /**
   * Parse category from text value to its int representation and set it into field.
   * @param category    String category (college, high_school, other)
   */
  public void parseAndSetCategory(String category) {
    if(category.equals("college")) {
      this.category = COLLEGE;
    } else if(category.equals("high_school")) {
      this.category = HIGH_SCHOOL;
    } else {
      this.category = OTHER;
    }
  }
  
  /**
   * Returns name of category from internal int representation.
   * @return category name
   */
  public String getCategoryName() {
    if(category == 0) {
      return "st\u0159edo\u0161kol\u00e1ci";
    } else if (category == 1) {
      return "vysoko\u0161kol\u00e1ci";
    } else {
      return "ostatn\u00ed";
    }
  }
}
/** @file Year.pde
 * @author Jan Drabek, Martin Ukrop
 * @brief holds data for 1 year, manages data area drawing
 */

class Year {
  String name;
  ArrayList<Team> teams = new ArrayList<Team>(); // Sorted by total score descending

  /** number of horisontl axws behind bars
   * determined from total score of the best team at the end
   */
  public int numAxes() {
    return ceil((float)teams.get(0).states[301].getTotalScore()/axesPointsInterval);
  }

  /** total number of teams in selected categories
   */
  public int numSelectedTeams() {
    int numTeams = 0;
    for (int i = 0; i < teams.size(); i++) {
      if (selectedCategories[teams.get(i).category]) numTeams++;
    } 
    return numTeams;
  }

  /** draw actual team data
   * to be done after background drawing and before scale drawing
   * calls individual temas's draw with correct offsets
   */
  public void drawData() {
    int dataWidth = numSelectedTeams()*(teamColumnWidth+teamColumnMargin)+teamColumnMargin;
    int dataShift;
    if (dataWidth + dataLeftMargin < screenWidth) {
      dataShift = 0;
    } else {
      dataShift = (int) map(dataSliderPosition, 0, 1, 0, dataWidth-screenWidth);
    }
    int renderedTeams = 0;
    for (int teamNum = 0; teamNum < teams.size(); teamNum++) {
      if (!selectedCategories[teams.get(teamNum).category]) continue;
      int x = renderedTeams*(teamColumnWidth+teamColumnMargin) + teamColumnMargin + dataLeftMargin - dataShift;
      teams.get(teamNum).draw(
        x,
        screenHeight-controlPanelHeight-dataBottomMargin, 
        numAxes()*axesPointsInterval, 
        screenHeight-controlPanelHeight-dataBottomMargin-dataTopMargin
      );
      if(currentTeamLeft == teamNum || currentTeamRight == teamNum) {
        if(currentTeamLeft == teamNum) {
          fill(green);
          stroke(green,155);
        }
        if(currentTeamRight == teamNum) {
          fill(red);
          stroke(red,155);
        }
        ellipse(x+teamColumnWidth/2, screenHeight-controlPanelHeight-dataBottomMargin+3, 10, 10);
        strokeWeight(2);
        float y = teams.get(teamNum).getLevel(
          screenHeight-controlPanelHeight-dataBottomMargin, 
          numAxes()*axesPointsInterval, 
          screenHeight-controlPanelHeight-dataBottomMargin-dataTopMargin
        ); 
        if (y != 0) line(dataLeftMargin, y, screenWidth, y);
        noStroke();
      }
      
      renderedTeams += 1;
    }
  }

  /** 
   * Draw data legend left from logos
   */  
  public void drawDataLegend() {
    int baseX = screenWidth - legendRightMargin;
    fill(red(brownLight), green(brownLight), blue(brownLight), 100);
    rect(baseX, legendTopMargin, 20,20);
        fill(red(brownLight), green(brownLight), blue(brownLight), 255);
    text("Logick\u00e9",baseX+30, legendTopMargin+textAscent()*1.3f);
 
    fill(red(brownLight), green(brownLight), blue(brownLight), 170);
    rect(baseX,legendTopMargin+30, 20,20);
    fill(red(brownLight), green(brownLight), blue(brownLight), 255);
    text("Programovac\u00ed",baseX+30, legendTopMargin+30+textAscent()*1.3f);
    
    fill(red(brownLight), green(brownLight), blue(brownLight), 255);
    rect(baseX, legendTopMargin+60, 20,20);
    text("\u0160ifrovac\u00ed", baseX+30, legendTopMargin+60+textAscent()*1.3f);
    
    fill(red(green), green(green), blue(green), 150);
    rect(baseX+180, legendTopMargin, 20,20);
    text("Bonus",baseX+210, legendTopMargin+textAscent()*1.3f);
    
    fill(red(red), green(red), blue(red), 150);
    rect(baseX+180, legendTopMargin+30, 20,20);
    text("Penalizace",baseX+210, legendTopMargin+30+textAscent()*1.3f);
  }

  /** background and horisontal axes in data area
   * puts background image
   * draws data lines
   * to be called before data drawing
   */
  public void drawDataBackground() {
    // place background
    int numImagesWidth = ceil((float)screenWidth/bgImage.width);
    int numImagesHeight = ceil((float)(screenHeight-controlPanelHeight)/bgImage.height);
    for (int j = 0; j < numImagesHeight; j++) {
      for (int i = 0; i < numImagesWidth; i++) {
        image(bgImage, i*bgImage.width, screenHeight-controlPanelHeight-(j+1)*bgImage.height);
      }
    }

    // draw horisontal axes
    stroke(red(brownLight), green(brownLight), blue(brownLight), 64);
    strokeWeight(axesStroke);
    for (int i = 0; i <= numAxes(); i++) {
      line (dataLeftMargin, map(i, 0, numAxes(), dataTopMargin, screenHeight-controlPanelHeight-dataBottomMargin), 
      screenWidth, map(i, 0, numAxes(), dataTopMargin, screenHeight-controlPanelHeight-dataBottomMargin));
    }
    noStroke();
  }

  /** draws vertical scale, title and logos
   * to be called after data drawing
   */
  public void drawDataAxes() {
    drawDataLegend();
    // place background to overdraw data parts on the left
    int numImagesHeight = ceil((float)(screenHeight-controlPanelHeight)/bgImage.height);
    for (int j = 0; j < numImagesHeight; j++) {
      image(bgImage, dataLeftMargin-bgImage.width, screenHeight-controlPanelHeight-(j+1)*bgImage.height);
    }
    
    // draw axis
    stroke(red(brownLight), green(brownLight), blue(brownLight), 64);
    strokeWeight(axesStroke);
    line(dataLeftMargin, screenHeight-controlPanelHeight-dataBottomMargin, dataLeftMargin, dataTopMargin-20);
    noStroke();
    
    // draw text labels on axis
    for (int i = 0; i <= numAxes(); i++) {
      String text = ""+i*axesPointsInterval;
      fill(brownDark);
      text(text, dataLeftMargin-textWidth(text)-10, 
      map(i, 0, numAxes(), screenHeight-controlPanelHeight-dataBottomMargin, dataTopMargin)+
        textAscent()/2);
    }

    // draw title + InterLoS logo
    fill(brownDark);
    textFont(fonts[2]);
    String title = "InterLoS " + name;
    text(title, logoImages[0].width+10, 3*dataTopMargin/5);
    float afterTitle = logoImages[0].width+10+textWidth(title);
    image(logoImages[0], 10, -10);
    
    // place university and faculty logo
    image(logoImages[2], screenWidth-logoImages[2].width-10, 10);
    image(logoImages[1], screenWidth-logoImages[2].width-10-logoImages[1].width-10, 10);

    // draw categories text
    textFont(fonts[0]);
    String categsText = "(";
    int numCategories = 0;
    if (selectedCategories[0] && selectedCategories[1] && selectedCategories[2]) {
      categsText += "v\u0161echny kategorie";
    } 
    else {
      if (selectedCategories[0]) { 
        categsText += "st\u0159edo\u0161kol\u00e1ci";
        numCategories++;
      }
      if (selectedCategories[1]) {
        if (numCategories > 0) { 
          categsText += ", ";
        } 
        categsText += "vysoko\u0161kol\u00e1ci";
        numCategories++;
      }
      if (selectedCategories[2]) { 
        if (numCategories > 0) { 
          categsText += ", ";
        } 
        categsText += "ostatn\u00ed";
        numCategories++;
      }
    }
    categsText += ")";
    text(categsText, afterTitle+10, 3*dataTopMargin/5);
  }

  /**
   * Draw team details boxes for currently selected teams.
   */  
  public void drawTeamInfo() {
    if(currentTeamLeft >= 0  && currentTeamLeft < teams.size()) {
      teams.get(currentTeamLeft).drawInfo(0, dataLeftMargin, screenHeight-controlPanelHeight-dataBottomMargin+teamDetailMargin);  
    }
    if(currentTeamRight >= 0  && currentTeamRight < teams.size()) {
      teams.get(currentTeamRight).drawInfo(450, dataLeftMargin, screenHeight-controlPanelHeight-dataBottomMargin+teamDetailMargin);  
    }
      
  }
  
  /** processes click in data area
   */
  public void processClick(int x, int y) {
    int baseX = dataLeftMargin;
    int baseY = screenHeight-controlPanelHeight-dataBottomMargin+teamDetailMargin;
    // Check closing click on team detail boxes cross
    if(isIn(x,baseX+415, baseX+415+25) &&
       isIn(y, baseY, baseY+25)) {
       currentTeamLeft = -1;
       redrawData = true;
       return;
    }
    if(isIn(x,baseX+415+450, baseX+415+450+25) &&
       isIn(y, baseY, baseY+25)) {
       currentTeamRight = -1;
       redrawData = true;
       return;
    }
    // Check click on team bar -> select this team
    int dataWidth = numSelectedTeams()*(teamColumnWidth+teamColumnMargin)+teamColumnMargin;
    int dataShift;
    if (dataWidth + dataLeftMargin < screenWidth) {
      dataShift = 0;
    } else {
      dataShift = (int) map(dataSliderPosition, 0, 1, 0, dataWidth-screenWidth);
    }
    int renderedTeams = 0;
    for (int teamNum = 0; teamNum < teams.size(); teamNum++) {
      if (!selectedCategories[teams.get(teamNum).category]) continue;
      int teamX = renderedTeams*(teamColumnWidth+teamColumnMargin) + teamColumnMargin + dataLeftMargin - dataShift;
      int teamY = screenHeight-controlPanelHeight-dataBottomMargin;
      if(isIn(x, teamX, teamX + teamColumnWidth) &&
         isIn(y, teamY - (screenHeight-controlPanelHeight-dataBottomMargin-dataTopMargin), teamY)) {
           if(mouseButton == LEFT) { // LEFT/RIGHT - for selecting two teams into two different areas
             currentTeamLeft = teamNum;
           } else if (mouseButton == RIGHT) {
             currentTeamRight = teamNum;
           }
           redrawData = true;
      }
      renderedTeams += 1;
    }
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "Visualisation" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
